
package magercises.minipim.solution;

/**
 * CalculatorKernel -- the "guts" of the calculator component
 * 
 */
public class CalculatorKernel {
	// A String property that represents the value that is displayed
	String fieldDisplay = "";
	
	// Support (generated automatically for bound properties) for
	//   catching changes to the display property.
	protected transient java.beans.PropertyChangeSupport propertyChange = new java.beans.PropertyChangeSupport(this);
	
	// pendingOperation keeps track of the operator key that was pressed
	// The "default" value for it is always "+".  This means that whenever
	//   the underlying memory is 0, any operator key that is pressed
	//   will set the memory += display (essentially just copying it)
	private String pendingOperation = "+";
	
	// The underlying memory of the calculator -- this is always the
	//   left operand of the expression being computed.  After equals or
	//   clear is pressed, this will be cleared.
	private float memory = 0;
	
	// A flag to tell us if we should append the next number pressed to
	//   the current display, or start the display fresh.   The display will
	//   always be started fresh when the clear, equals or an operation
	//   key is pressed.
	private boolean startNewDisplay = true;

/**
 * The addPropertyChangeListener method was generated to support the propertyChange field.
 */
public synchronized void addPropertyChangeListener(java.beans.PropertyChangeListener listener) {
	propertyChange.addPropertyChangeListener(listener);
}
/** compute -- determine the proper settings for the display and memory
 *  Note that a four-function calculator is not as straightforward as
 *    it may at first seem.  When an operation key is pressed, the
 *    computation performed is based on the _last_ operation pressed.
 *    (This is because the expression that the user is currently
 *     typing is in _infix_ notation.  That is, they have only typed the
 *     left operand and the operator.  The number in the display is
 *     actually finalizing the _previous_ computation...)
 */
	public void compute() {
		float displayValue;
		
		// convert the String display value to a real float...
		// there are three special values to watch for
		//   Infinity -- a positive number / 0
		//   -Infinity -- a negative number / 0
		//   NaN      -- something goofy like 0 / 0
		// make sure we properly assign these values
		if (getDisplay().equals("Infinity"))
			displayValue = Float.POSITIVE_INFINITY;
		else if (getDisplay().equals("-Infinity"))
			displayValue = Float.NEGATIVE_INFINITY;
		else if (getDisplay().equals("NaN"))
			displayValue = Float.NaN;
		else	
			displayValue = Float.valueOf(getDisplay()).floatValue();
		
		// take a look at what the _last_ operation key pressed is
		// note that if we had finished the last calculation by
		//   pressing clear or equals, pendingOperation would have been
		//   set to "+" and memory would be set to 0...
		switch (pendingOperation.charAt(0)) {
			case '-':
				memory -= displayValue;
				break;
				
			case 'X':
				memory *= displayValue;
				break;
				
			case '/':
				// Java treats divide by zero as "Infinity"  try
				//   dividing by zero to see what it does...
				memory /= displayValue;
				break;

			default: // + or anything else
			         // anything else will just set mem to display
				memory += displayValue;
				break;				
		}
		setDisplay(Float.toString(memory));
		pendingOperation = "+";
	}	
/**
 * The firePropertyChange method was generated to support the propertyChange field.
 */
public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
	propertyChange.firePropertyChange(propertyName, oldValue, newValue);
}
/**
 * Gets the display property (java.lang.String) value.
 * @return The display property value.
 * @see #setDisplay
 */
public String getDisplay() {
	/* Returns the display property value. */
	if (fieldDisplay == null) {
		try {
			fieldDisplay = new String();
		} catch (Throwable exception) {
			System.err.println("Exception creating display property.");
		}
	};
	return fieldDisplay;
}
/**
 * Performs the key method.
 * @param keyName java.lang.String
 */
public void key(String keyName) {
	/* Perform the key method. */
	switch (keyName.charAt(0)) {
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
		case '.':
			String newDisplay;
			if (startNewDisplay || getDisplay().equals("0")) {
				newDisplay = keyName;
				startNewDisplay = false;
			}	
			else newDisplay = getDisplay()+keyName;
			setDisplay(newDisplay);
			break;
			
		case 'C':
			setDisplay("0");
			startNewDisplay = true;
			memory = 0;
			pendingOperation = "+";
			break;
			
		case '+':
		case '-':
		case 'X':
		case '/':
		case '=':
			compute();
			pendingOperation = keyName;
			startNewDisplay = true;
			if (keyName.equals("="))
				memory = 0;
			break;
	}	
	return;
}
/**
 * The removePropertyChangeListener method was generated to support the propertyChange field.
 */
public synchronized void removePropertyChangeListener(java.beans.PropertyChangeListener listener) {
	propertyChange.removePropertyChangeListener(listener);
}
/**
 * Sets the display property (java.lang.String) value.
 * @param display The new value for the property.
 * @see #getDisplay
 */
public void setDisplay(String display) {
	/* Get the old property value for fire property change event. */
	String oldValue = fieldDisplay;
	/* Set the display property (attribute) to the new value. */
	fieldDisplay = display;
	/* Fire (signal/notify) the display property change event. */
	firePropertyChange("display", oldValue, display);
	return;
}
}